@@ -12,7 +12,7 @@ use Params::Validate  qw/validate SCALAR SCALARREF CODEREF ARRAYREF HASHREF
 use Scalar::Util      qw/blessed reftype/;
 use Carp;
 
-our $VERSION = '1.27';
+our $VERSION = '1.28';
 
 # import the "puke" function from SQL::Abstract (kind of "die")
 BEGIN {*puke = \&SQL::Abstract::puke;}
@@ -34,9 +34,6 @@ sub does ($$) {
 }
 
 
-
-
-
 #----------------------------------------------------------------------
 # remove all previously defined functions
 #----------------------------------------------------------------------
@@ -90,16 +87,17 @@ s/JOIN %s/JOIN (%s)/ foreach values %right_assoc_join_syntax;
 
 # specification of parameters accepted by the new() method
 my %params_for_new = (
-  table_alias          => {type => SCALAR|CODEREF, default  => '%s AS %s'},
-  column_alias         => {type => SCALAR|CODEREF, default  => '%s AS %s'},
-  limit_offset         => {type => SCALAR|CODEREF, default  => 'LimitOffset'},
-  join_syntax          => {type => HASHREF,        default  =>
+  table_alias          => {type => SCALAR|CODEREF,   default  => '%s AS %s'},
+  column_alias         => {type => SCALAR|CODEREF,   default  => '%s AS %s'},
+  limit_offset         => {type => SCALAR|CODEREF,   default  => 'LimitOffset'},
+  join_syntax          => {type => HASHREF,          default  =>
                                                         \%common_join_syntax},
-  join_assoc_right     => {type => BOOLEAN,        default  => 0},
-  max_members_IN       => {type => SCALAR,         optional => 1},
+  join_assoc_right     => {type => BOOLEAN,          default  => 0},
+  max_members_IN       => {type => SCALAR,           optional => 1},
   multicols_sep        => {type => SCALAR|SCALARREF, optional => 1},
-  has_multicols_in_SQL => {type => BOOLEAN,        optional => 1},
-  sql_dialect          => {type => SCALAR,         optional => 1},
+  has_multicols_in_SQL => {type => BOOLEAN,          optional => 1},
+  sql_dialect          => {type => SCALAR,           optional => 1},
+  select_implicitly_for=> {type => SCALAR|UNDEF,     optional => 1},
 );
 
 # builtin collection of parameters, for various databases
@@ -181,20 +179,25 @@ sub new {
     $more_params{$_} ||= $dialect_params->{$_} foreach keys %$dialect_params;
   }
 
-  # check parameters
+  # check parameters for this class
   my @more_params = %more_params;
   my $more_self   = validate(@more_params, \%params_for_new);
 
+  # check some of the params for parent -- because SQLA doesn't do it :-(
+  !$params{quote_char} || exists $params{name_sep}
+    or carp "when 'quote_char' is present, 'name_sep' should be present too";
+  # TODO : validate(%params)
+
   # call parent constructor
   my $self = $class->next::method(%params);
-    # TODO - validate %params -- because SQLA doesn't do it :-(
+
 
   # inject into $self
   $self->{$_} = $more_self->{$_} foreach keys %$more_self;
 
   # arguments supplied as scalars are transformed into coderefs
-  ref $self->{column_alias} or $self->_make_AS_through_sprintf('column_alias');
-  ref $self->{table_alias}  or $self->_make_AS_through_sprintf('table_alias');
+  ref $self->{column_alias} or $self->_make_sub_column_alias;
+  ref $self->{table_alias}  or $self->_make_sub_table_alias;
   ref $self->{limit_offset} or $self->_choose_LIMIT_OFFSET_dialect;
 
   # regex for parsing join specifications
@@ -331,8 +334,9 @@ sub select {
     push @bind, @limit_bind;
   }
 
-  # add FOR if needed
-  $sql .= " FOR $args{-for}" if $args{-for};
+  # add FOR clause if needed
+  my $for = exists $args{-for} ? $args{-for} : $self->{select_implicitly_for};
+  $sql .= " FOR $for" if $for;
 
   if ($args{-want_details}) {
     return {sql             => $sql,
@@ -466,8 +470,6 @@ foreach my $attr (qw/table_alias column_alias limit_offset/) {
   };
 }
 
-
-
 # readonly accessor methods
 foreach my $key (qw/join_syntax  join_assoc_right
                     max_members_IN multicols_sep  has_multicols_in_SQL/) {
@@ -476,9 +478,6 @@ foreach my $key (qw/join_syntax  join_assoc_right
 }
 
 
-
-
-
 # invocation method for 'join'
 sub join {
   my $self = shift;
@@ -641,8 +640,8 @@ sub _parse_join_spec {
       or croak "can't parse join condition: $cond";
 
     # if operands are not qualified by table/alias name, add sprintf hooks
-    $left  = "%1\$s.$left"  unless $left  =~ /\./;
-    $right = "%2\$s.$right" unless $right =~ /\./ or $right eq $placeholder;
+    $left  = '%1$s.' . $left   unless $left  =~ /\./;
+    $right = '%2$s.' . $right  unless $right =~ /\./ or $right eq $placeholder;
 
     # add this pair into the list; right operand is either a bind value
     # or an identifier within the right table
@@ -665,15 +664,24 @@ sub _single_join {
   @_ = reverse @_ if $self->{join_assoc_right};
   my ($left, $join_spec, $right) = @_;
 
-  # compute the "ON" clause (assuming it contains '%1$s', '%2$s' for
-  # left/right tables)
+  # compute the "ON" clause
   my ($sql, @bind) = $self->where($join_spec->{condition});
   $sql =~ s/^\s*WHERE\s+//;
-  $sql = sprintf $sql, $left->{name}, $right->{name};
 
-  # assemble all elements
+  # syntax for assembling all elements
   my $syntax = $self->{join_syntax}{$join_spec->{operator}};
-  $sql = sprintf $syntax, $left->{sql}, $right->{sql}, $sql;
+
+  # $sql may _intentionally_ omit %.. parameters, so disable warnings
+  { no if $] ge '5.022000', warnings => 'redundant';
+
+    # substitute left/right tables names for '%1$s', '%2$s'
+    $sql = sprintf $sql, $left->{name}, $right->{name};
+
+    # build the final sql
+    $sql = sprintf $syntax, $left->{sql}, $right->{sql}, $sql;
+  }
+
+  # add left/right bind parameters (if any) into the list
   unshift @bind, @{$left->{bind}}, @{$right->{bind}};
 
   # build result and return
@@ -994,19 +1002,47 @@ sub _overridden_update {
   return wantarray ? ($sql, @all_bind) : $sql;
 }
 
+
+
 #----------------------------------------------------------------------
 # method creations through closures
 #----------------------------------------------------------------------
 
-sub _make_AS_through_sprintf {
-  my ($self, $attribute) = @_;
-  my $syntax = $self->{$attribute};
-  $self->{$attribute} = sub {
+sub _make_sub_column_alias {
+  my ($self) = @_;
+  my $syntax = $self->{column_alias};
+  $self->{column_alias} = sub {
     my ($self, $name, $alias) = @_;
-    return $alias ? sprintf($syntax, $name, $alias) : $name;
+    return $name if !$alias;
+
+    # quote $name unless it is an SQL expression (then the user should quote it)
+    $name = $self->_quote($name) unless $name =~ /[()]/;
+
+    # assemble syntax
+    my $sql = sprintf $syntax, $name, $self->_quote($alias);
+
+    # return a string ref to avoid quoting by SQLA
+    return \$sql;
   };
 }
 
+
+sub _make_sub_table_alias {
+  my ($self) = @_;
+  my $syntax = $self->{table_alias};
+  $self->{table_alias} = sub {
+    my ($self, $name, $alias) = @_;
+    return $name if !$alias;
+
+    # assemble syntax
+    my $sql = sprintf $syntax, $self->_quote($name), $self->_quote($alias);
+
+    return $sql;
+  };
+}
+
+
+
 sub _choose_LIMIT_OFFSET_dialect {
   my $self = shift;
   my $dialect = $self->{limit_offset};
@@ -1273,6 +1309,20 @@ automatically converted using boolean logic :
 If the flag is false, subqueries are not allowed.
 
 
+=item select_implicitly_for
+
+A value that will be automatically added as a C<-for> clause in
+calls to L</select>. This default clause can always be overridden
+by an explicit C<-for> in a given select :
+
+  my $sqla = SQL::Abstract->new(-select_implicitly_for => 'READ ONLY');
+  ($sql, @bind) = $sqla->select(-from => 'Foo');
+    # SELECT * FROM FOO FOR READ ONLY
+  ($sql, @bind) = $sqla->select(-from => 'Foo', -for => 'UPDATE');
+    # SELECT * FROM FOO FOR UPDATE
+  ($sql, @bind) = $sqla->select(-from => 'Foo', -for => undef);
+    # SELECT * FROM FOO
+
 =item sql_dialect
 
 This is actually a "meta-argument" : it injects a collection
@@ -1372,6 +1422,21 @@ using perl C<< qw/.../ >> operator for columns, as in
   -columns => [ qw/table1.longColumn|t1lc table2.longColumn|t2lc/ ]
 
 SQL column aliasing is then generated through the L</column_alias> method.
+If L<SQL::Abstract/quote_char> is defined, aliased columns will be quoted,
+unless they contain parentheses, in which case they are considered as
+SQL expressions for which the user should handle the quoting himself.
+For example if C<quote_char> is "`", 
+
+  -columns => [ qw/foo.bar|fb length(buz)|lbuz/ ]
+
+will produce
+
+  SELECT `foo`.`bar` AS fb, length(buz) AS lbuz
+
+and not
+
+  SELECT `foo`.`bar` AS fb, length(`buz`) AS lbuz
+
 
 Initial items in C<< @columns >> that start with a minus sign
 are shifted from the array, i.e. they are not considered as column
@@ -1914,45 +1979,9 @@ L</bind_params> from the present module to perform the appropriate
 bindings before executing the statement.
 
 
-=head1 TODO
-
-Future versions may include some of these features :
-
-=over
-
-=item *
-
-support for C<WITH> initial clauses, and C<WITH RECURSIVE>.
-
-=item *
-
-support for Oracle-specific syntax for recursive queries
-(START_WITH, PRIOR, CONNECT_BY NOCYCLE, CONNECT SIBLINGS, etc.)
-
-=item *
-
-support for INSERT variants
-
-    INSERT .. DEFAULT VALUES
-    INSERT .. VALUES(), VALUES()
-
-=item *
-
-support for MySQL C<LOCK_IN_SHARE_MODE>
-
-=item *
-
-new constructor option 
-
-  ->new(..., select_implicitly_for => $string, ...)
-
-This would provide a default values for the C<-for> parameter.
-
-=back
-
 =head1 AUTHOR
 
-Laurent Dami, C<< <laurent.dami at justice.ge.ch> >>
+Laurent Dami, C<< <laurent dot dami at cpan dot org> >>
 
 =head1 BUGS
 
@@ -1993,7 +2022,7 @@ L<https://metacpan.org/module/SQL::Abstract::More>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011-2015 Laurent Dami.
+Copyright 2011-2016 Laurent Dami.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
